function z_dot = ffun_s_linear(x,tau,d)
% ODE-function implementing Cybership II model. Used for estimating states
% with given input and current. Assumes 2. order modulus damping model.
% 
%
% x_dot = ffun_s(x,tau,d)
%
% INPUTS:
% x - complete state of ship. 
%   x(1:3) = eta  
%   x(4:6) = nu
% d(1:2) = [x_cdot; y_cdot]
% d(3:5) = [b_x; b_y; b_psi]
% tau - force input
%
% OUTPUT:
% x_dot - the value of differential equation

global cs

A = cs.m - cs.X_udot;
C = cs.m - cs.Y_vdot;
D = cs.m*cs.x_G - cs.Y_rdot;
F = cs.m*cs.x_G - cs.N_vdot;
G = cs.I_z - cs.N_rdot;
L = C*G - D*F;

[n_x,n_s] = size(x);
z_dot = zeros(n_x,n_s);


% Current is identical for all parallells
x_cdot = d(1);
y_cdot = d(2);
b_x = d(3);
b_y = d(4);
b_psi = d(5);


for k = 1:n_s
psi = x(3,k);
u = x(4,k);
v = x(5,k);
r = x(6,k);

% Rotasjonsmatrise
c_psi = cos(psi);
s_psi = sin(psi);

u_c = c_psi*x_cdot + s_psi*y_cdot;
v_c = -s_psi*x_cdot + c_psi*y_cdot;

% Relative speed
u_r = u - u_c;
v_r = v - v_c;

% Deriverte
u_dot = (tau(1) + cs.X3_u*u_r + c_psi*b_x + s_psi*b_y)/A;  

E = (tau(2) + cs.Y3_r*r + cs.Y3_v*v_r - s_psi*b_x + c_psi*b_y);

H = (tau(3) + cs.N3_v*v_r + cs.N3_r*r + b_psi);
          
v_dot = (G*E-D*H)/L;
r_dot = (C*H-F*E)/L;

x_dot = c_psi*u - s_psi*v;
y_dot = s_psi*u + c_psi*v;
psi_dot = r;


z_dot(:,k) = [x_dot; y_dot; psi_dot; u_dot; v_dot; r_dot];
end
end

  
        
        