//  ODE-function implementing Cybership II model. Used for estimating states
//  with given input, parameters and current. Assumes 2. order modulus damping model.
//  Current given with speed and angle
//
//  x_dot = ffun_ukf_s(x,tau,d)
//
// INPUTS:
//  x - complete state of ship.
//    x(1:3) = eta
//    x(4:6) = nu
//  d(1:2) = eta_cdot
//  d(3:6) = parameters
//  d(7:9) = [b_x; b_y; b_psi]
//  tau - force input
//
//  OUTPUT:
//  x_dot - the value of differential equation

#include "mex.h"
#include "matrix.h"
#include "math.h"




void mexFunction(int nOut, mxArray *pOut[], int nIn, const mxArray *pIn[]) {
    double m, I_z, x_G, X_udot, Y_vdot, Y_rdot, N_vdot, N_rdot;
    double X2_u, Y2_v, Y2_r, N2_v, N2_r, X2_uu, X2_uuu, Y2_vvv, Y2_rrr, N2_vvv, N2_rrr;
    double Y2_vvr, Y2_vrr, N2_vvr, N2_vrr;
    double A, C, D, E, F, G, H, L;
    double u, v, r, psi, x_cdot, y_cdot, u_c, v_c, v_r, u_r, u_cdot, v_cdot, c_psi, s_psi;
    double* x;
    double* d;
    double* tau;
    double x_dot, y_dot, psi_dot, u_dot, v_dot, r_dot, b_x, b_y, b_psi;
    double* z_dot;
    double* p;
    mxArray* cs;
    mxArray* n;
    int k,n_x,n_s,n_u,n_d;
    
    
    //     Check parameters
    if (nOut != 1 || nIn != 3) {
        mexErrMsgTxt("Invalid params for 'ffun_s'");
        return;
    }
    
    //     Fetch input parameters
    x = mxGetPr(pIn[0]);
    tau = mxGetPr(pIn[1]);
    d = mxGetPr(pIn[2]);
    
    n_x = mxGetM(pIn[0]);
    n_s = mxGetN(pIn[0]);
    n_d = mxGetM(pIn[1]);
    n_u = mxGetM(pIn[2]);
    
    //     Output matrix
    z_dot = (double*) malloc(sizeof(double)*n_x*n_s);
    
    
    //     Unpack cybership parameters
    cs = mexGetVariable("global", "cs");
    m = mxGetScalar(mxGetField(cs, 0, "m"));
    I_z = mxGetScalar(mxGetField(cs, 0, "I_z"));
    x_G = mxGetScalar(mxGetField(cs, 0, "x_G"));
    X_udot = mxGetScalar(mxGetField(cs, 0, "X_udot"));
    Y_vdot = mxGetScalar(mxGetField(cs, 0, "Y_vdot"));
    Y_rdot = mxGetScalar(mxGetField(cs, 0, "Y_rdot"));
    N_vdot = mxGetScalar(mxGetField(cs, 0, "N_vdot"));
    N_rdot = mxGetScalar(mxGetField(cs, 0, "N_rdot"));
    
    Y2_v = mxGetScalar(mxGetField(cs, 0, "Y2_v"));
    Y2_r = mxGetScalar(mxGetField(cs, 0, "Y2_r"));
    N2_v = mxGetScalar(mxGetField(cs, 0, "N2_v"));
    N2_r = mxGetScalar(mxGetField(cs, 0, "N2_r"));
    
    X2_u = mxGetScalar(mxGetField(cs, 0, "X2_u"));
    X2_uuu = mxGetScalar(mxGetField(cs, 0, "X2_uuu"));
    Y2_vvv = mxGetScalar(mxGetField(cs, 0, "Y2_vvv"));
    Y2_rrr = mxGetScalar(mxGetField(cs, 0, "Y2_rrr"));
    N2_vvv = mxGetScalar(mxGetField(cs, 0, "N2_vvv"));
    N2_rrr = mxGetScalar(mxGetField(cs, 0, "N2_rrr"));
    
    Y2_vvr = d[2];
    Y2_vrr = d[3];
    N2_vvr = d[4];
    N2_vrr = d[5];
    b_x = d[6];
    b_y = d[7];
    b_y = d[8];
    
    A = m - X_udot;
    C = m - Y_vdot;
    D = m*x_G - Y_rdot;
    F = m*x_G - N_vdot;
    G = I_z - N_rdot;
    L = C*G - D*F;
    
    //     Current is same for all sigmapoints
    x_cdot = d[0];
    y_cdot = d[1];
    
    for(k=0; k < n_s; k++) {
        psi = x[2 + k*n_x];
        u = x[3 + k*n_x];
        v = x[4 + k*n_x];
        r = x[5 + k*n_x];
        
        
        //     % Rotasjonsmatrise
        c_psi = cos(psi);
        s_psi = sin(psi);
        
        u_c = c_psi*x_cdot + s_psi*y_cdot;
        v_c = -s_psi*x_cdot + c_psi*y_cdot;
        
        u_cdot = r*v_c;
        v_cdot = -r*u_c;
        
        //     % Relative speed
        u_r = u - u_c;
        v_r = v - v_c;
        
        //     % Deriverte
        u_dot = (tau[0] + X_udot*u_cdot + m*(x_G*r + v)*r - (Y_vdot*v_r + (N_vdot + Y_rdot)*r/2)*r + X2_u*u_r + X2_uuu*u_r*u_r*u_r + c_psi*b_x + s_psi*b_y)/A;
        
        E = (tau[1] + Y_vdot*v_cdot - m*u*r + X_udot*u_r*r + Y2_v*v_r + Y2_r*r + Y2_vvv*v_r*v_r*v_r + Y2_vvr*v_r*v_r*r + Y2_vrr*v_r*r*r + Y2_rrr*r*r*r - s_psi*b_x + c_psi*b_y);
        
        H = (tau[2] + N_vdot*v_cdot - m*x_G*r*u + (Y_vdot*v_r + (N_vdot + Y_rdot)*r/2)*u_r - X_udot*u_r*v_r + N2_v*v_r + N2_r*r + N2_vvv*v_r*v_r*v_r + N2_vvr*v_r*v_r*r + N2_vrr*v_r*r*r + N2_rrr*r*r*r + b_psi);
        
        v_dot = (G*E-D*H)/L;
        r_dot = (C*H-F*E)/L;
        
        x_dot = c_psi*u - s_psi*v;
        y_dot = s_psi*u + c_psi*v;
        psi_dot = r;
        
        z_dot[0 + k*n_x] = x_dot;
        z_dot[1 + k*n_x] = y_dot;
        z_dot[2 + k*n_x] = psi_dot;
        z_dot[3 + k*n_x] = u_dot;
        z_dot[4 + k*n_x] = v_dot;
        z_dot[5 + k*n_x] = r_dot;
    }
    n = mxCreateDoubleMatrix(n_x,n_s, mxREAL);
    p = mxGetPr(n);
    memcpy(p,z_dot,sizeof(double)*n_x*n_s);
    free(z_dot);
    pOut[0] = n;
}