// ODE-function implementing Cybership II model. Uses 2. order modulus
// damping. Estimates states and current. 
//
//  z_dot = ffun_ukf_sd(x,tau,d)
//
// INPUTS:
//  x - complete state of ship.
//    x(1:3) = eta
//    x(4:6) = nu
//    x(7:8) = [x_cdot; y_cdot]
//  tau - force input
//  d(1:3) = [b_x; b_y; b_psi]
//  OUTPUT:
//  z_dot - the value of differential equation

#include "mex.h"
#include "matrix.h"
#include "math.h"




void mexFunction(int nOut, mxArray *pOut[], int nIn, const mxArray *pIn[]) {
    double m, I_z, x_G, X_udot, Y_vdot, Y_rdot, N_vdot, N_rdot;
    double X_u, Y_v, Y_r, N_v, N_r, X_uu, X_uuu, Y_vv, Y_rr, N_vv, N_rr;
    double Y_vr, Y_rv, N_vr, N_rv;
    double A, C, D, E, F, G, H, L;
    double u, v, r, psi, x_cdot, y_cdot, u_c, v_c, v_r, u_r, u_cdot, v_cdot, c_psi, s_psi;
    double* x;
    double* d;
    double* tau;
    double x_dot, y_dot, psi_dot, u_dot, v_dot, r_dot, b_x, b_y, b_psi;
    double* z_dot;
    double* p;
    mxArray* cs;
    mxArray* n;
    int k,n_x,n_s;
    
    
    //     Check parameters
    if (nOut != 1 || nIn != 3) {
        mexErrMsgTxt("Invalid params for 'ffun_sd'. Should be one output and three inputs");
        return;
    }
    
    //     Fetch input parameters
    x = mxGetPr(pIn[0]);
    tau = mxGetPr(pIn[1]);
    d = mxGetPr(pIn[2]);
    
    n_x = mxGetM(pIn[0]);
    n_s = mxGetN(pIn[0]);
    
    //     Output matrix
    z_dot = (double*) malloc(sizeof(double)*n_x*n_s);
    
    
    //     Unpack cybership parameters
    cs = mexGetVariable("global", "cs");
    m = mxGetScalar(mxGetField(cs, 0, "m"));
    I_z = mxGetScalar(mxGetField(cs, 0, "I_z"));
    x_G = mxGetScalar(mxGetField(cs, 0, "x_G"));
    X_udot = mxGetScalar(mxGetField(cs, 0, "X_udot"));
    Y_vdot = mxGetScalar(mxGetField(cs, 0, "Y_vdot"));
    Y_rdot = mxGetScalar(mxGetField(cs, 0, "Y_rdot"));
    N_vdot = mxGetScalar(mxGetField(cs, 0, "N_vdot"));
    N_rdot = mxGetScalar(mxGetField(cs, 0, "N_rdot"));
    
    Y_v = mxGetScalar(mxGetField(cs, 0, "Y_v"));
    Y_r = mxGetScalar(mxGetField(cs, 0, "Y_r"));
    N_v = mxGetScalar(mxGetField(cs, 0, "N_v"));
    N_r = mxGetScalar(mxGetField(cs, 0, "N_r"));
    
    X_u = mxGetScalar(mxGetField(cs, 0, "X_u"));
    X_uu = mxGetScalar(mxGetField(cs, 0, "X_uu"));
    X_uuu = mxGetScalar(mxGetField(cs, 0, "X_uuu"));
    Y_vv = mxGetScalar(mxGetField(cs, 0, "Y_vv"));
    Y_rr = mxGetScalar(mxGetField(cs, 0, "Y_rr"));
    Y_vr = mxGetScalar(mxGetField(cs, 0, "Y_vr"));
    Y_rv = mxGetScalar(mxGetField(cs, 0, "Y_rv"));
    N_vv = mxGetScalar(mxGetField(cs, 0, "N_vv"));
    N_rr = mxGetScalar(mxGetField(cs, 0, "N_rr"));
    N_vr = mxGetScalar(mxGetField(cs, 0, "N_vr"));
    N_rv = mxGetScalar(mxGetField(cs, 0, "N_rv"));
    
    A = m - X_udot;
    C = m - Y_vdot;
    D = m*x_G - Y_rdot;
    F = m*x_G - N_vdot;
    G = I_z - N_rdot;
    L = C*G - D*F;
    
    b_x = d[0];
    b_y = d[1];
    b_psi = d[2];
    
    for(k=0; k < n_s; k++) {
        psi = x[2 + k*n_x];
        u = x[3 + k*n_x];
        v = x[4 + k*n_x];
        r = x[5 + k*n_x];
        x_cdot = x[6 + k*n_x];
        y_cdot = x[7 + k*n_x];
        
        
        //     % Rotasjonsmatrise
        c_psi = cos(psi);
        s_psi = sin(psi);
        
        u_c = c_psi*x_cdot + s_psi*y_cdot;
        v_c = -s_psi*x_cdot + c_psi*y_cdot;
        
        u_cdot = r*v_c;
        v_cdot = -r*u_c;
        
        //     % Relative speed
        u_r = u - u_c;
        v_r = v - v_c;
        
        //     % Deriverte
        u_dot = (tau[0] - X_udot*u_cdot + m*(x_G*r + v)*r - (Y_vdot*v_r + (N_vdot + Y_rdot)*r/2)*r + (X_u + X_uu*fabs(u_r) + X_uuu*u_r*u_r)*u_r + c_psi*b_x + s_psi*b_y)/A;
        
        E = tau[1] - Y_vdot*v_cdot - m*u*r + X_udot*u_r*r + Y_v*v_r + Y_r*r + (Y_vv*fabs(v_r) + Y_rv*fabs(r))*v_r + (Y_vr*fabs(v_r) + Y_rr*fabs(r))*r - s_psi*b_x + c_psi*b_y;
        
        H = tau[2] - N_vdot*v_cdot - m*x_G*r*u + (Y_vdot*v_r + (N_vdot + Y_rdot)*r/2)*u_r - X_udot*u_r*v_r + N_v*v_r + N_r*r + (N_vv*fabs(v_r) + N_rv*fabs(r))*v_r + (N_vr*fabs(v_r) + N_rr*abs(r))*r + b_psi;
        
        v_dot = (G*E-D*H)/L;
        r_dot = (C*H-F*E)/L;
        
        x_dot = c_psi*u - s_psi*v;
        y_dot = s_psi*u + c_psi*v;
        psi_dot = r;
        
        
        z_dot[0 + k*n_x] = x_dot;
        z_dot[1 + k*n_x] = y_dot;
        z_dot[2 + k*n_x] = psi_dot;
        z_dot[3 + k*n_x] = u_dot;
        z_dot[4 + k*n_x] = v_dot;
        z_dot[5 + k*n_x] = r_dot;
        z_dot[6 + k*n_x] = 0;
        z_dot[7 + k*n_x] = 0;
    }
    n = mxCreateDoubleMatrix(n_x,n_s, mxREAL);
    p = mxGetPr(n);
    memcpy(p,z_dot,sizeof(double)*n_x*n_s);
    free(z_dot);
    pOut[0] = n;
}