function [waveForceTotal,waveForceDrift,waveForceFK] = waveForces(t,waveDir,init,DOF,shortCrest,tFinal)
% Calculates disturbanceforce from 1. order and 2. order wave disturbances.
% Given in BODY-frame.
% [waveForceTotal,waveForceDrift,waveForceFK] = waveForces(t,waveDir,init,DOF,shortCrest,tFinal)
%
% INPUT:
% t = time
% waveDir = main wavedirection
% init = set to 0 to run init (select random variables and such)
% DOF = set to 3 if 3DOF horizontal model is used (surge, sway, yaw).
% Otherwise 6DOF is used
% shortCrest = set to 1 to calculate shortcrested waves
% tFinal = used to make sure that the random waves have a period of the
% same length as the simulation time. Is used to decide number of
% wave-components
%
% OUTPUT:
% waveForcetotal = vector of total force and moment
% waveForceDrift = force due to 2.order wave disturbance
% waveForceFK = force due to 1.order wave disturbance


persistent  ro g Lpp B draught V S wStep W N M chiStep Chi chi epsilon wi kn n

% specter = 'ISSC';

if ~init

    %% Initial parameters
    %%Parameters
    ro = 1025;
    g = 9.80665;
    % Ship parameters
    Lpp = 87.8500;%68.2;
    B = 20.3;%17.2;
    draught = 5;%6.3;
    V = 4680;%5311;


    %% Generate wave spectrum
    wMin = 0.1; wMax = 4; % Wavefrequencies
    wStep = 2*pi/tFinal; % Smaller steps, better resolution but slower simulation. Waves will repeat every t = 2*pi/wStep seconds
    W = (wMin:wStep:wMax)';
    N = length(W);


    % Wave parameters
    Hs = 1.8;% significant wave height
    T0 = 7;% modal wave period
    % W0 = 2*pi/T0; 
    % gamma = 3.3; % JONSWAP-spectrum only
    % wMin = pi/T0;  % PM-spectrum only
    % Calculate wave spectrum
    S = wavespec(3,[Hs,T0],W,0)'; % ISSC MPM wave spectrum. (1/0) = Plot/No plot;
    % S = wavespec(7,[Hs, W0, gamma],W,0)'; % JONSWAP wave spectrum
    % S = wavespec(8,[Hs,W0],W,1); % Torsethaugen wave spectrum


    if shortCrest
        M = 8; % Fewer steps = less power = number of directions
        chiStep = 2*pi/(M - 1);
        Chi = (-pi:chiStep:pi)';
        n = 2; % Power spreading function
        s_chi = 22000;
        randn('seed',s_chi);
        rand_chi = randn(M,1);
        chi = Chi + chiStep.*(rand_chi-1/2);
    else
        M = 1;
    end


    rand_wi = rand(N,1);
    epsilon = rand(N,1)*2*pi;

    s_w = 25000;
    rand('seed',s_w);
    wi = W + wStep.*(rand_wi-1/2);   % random frequency in [w-wStep/2,w+wStep/2])
    kn = wi.^2/g; % Deep water. Wave number calculation

end

if shortCrest
    Sspread = spreadingWaveSpec(S,W,chi,waveDir,n,0);
end

waveEl = 0;
waveAcc1 = 0;
for kk = 1:N
    for ll = 1:M
        if shortCrest
            Swichi = interp2(W,Chi,Sspread,wi(kk),rad2pipi(chi(ll)-waveDir),'nearest');
            A = sqrt(2*wStep*chiStep*Swichi);
        else
            Swi = interp1(W,S,wi(kk),'cubic');
            A = sqrt(2*Swi*wStep);
        end

        if imag(A) ~= 0
            disp('Trbbel i trnet')
        end
        waveEl = waveEl + A*sin(wi(kk)*t + epsilon(kk));
        waveAcc1 = waveAcc1 + wi(kk)^2*A*exp(-kn(kk)*draught)*cos(wi(kk)*t + epsilon(kk));
    end
end

waveAcc3 = -waveAcc1;

waveForceDrift(1) = 1/8*ro*g*waveEl^2*B*cos(waveDir); 
waveForceDrift(2) = 1/8*ro*g*waveEl^2*Lpp*sin(waveDir);
waveForceFK(1) = ro*V*cos(waveDir)*waveAcc1;
waveForceFK(2) = ro*V*sin(waveDir)*waveAcc1;


if DOF == 3
    waveForceDrift(3) = 1/32*ro*g*waveEl^2*Lpp*draught; %Not entirely correct, hacked to get more accurate size
    waveForceFK(3) = ro*V*waveAcc3*draught;%Not entirely correct, hacked to get more accurate size
else
    waveForceDrift(3) = 1/8*ro*g * waveEl^2*B*Lpp;
    waveForceDrift(4) = 1/8*ro*g * waveEl^2*B*cos(waveDir)*B;
    waveForceDrift(5) = 1/8*ro*g * waveEl^2*Lpp*sin(waveDir)*Lpp;
    waveForceDrift(6) = 1/8*ro*g * waveEl^2*Lpp*draught; %Not entirely correct, hacked to get more accurate size

    waveForceFK(3) = ro*V*waveAcc3;
    waveForceFK(4) = ro*V*cos(waveDir)*waveAcc1*B;
    waveForceFK(5) = ro*V*sin(waveDir)*waveAcc1*Lpp;
    waveForceFK(6) = ro*V*waveAcc3*draught/10;%Not entirely correct, hacked to get more accurate size
end
waveForceTotal = (waveForceDrift + waveForceFK)';