% WAVESEAWAY
% Time-series realization of waves in a seaway (surge).
%
% Currently supported:  - Spreading spectrum: short-crested seas
%                       wave elevation time series,
%                       wave force time series (2nd-order wave drift)
%                                              (1st-order Froude-Krilov)
% Wave drift: 1/2*ro*g*Amplitude^2*length
% Froude-Krilov; ro*Volume*waveAcc
%
% Forces/moments are typically stronger for longer-crested seas.
%
% Based on O. M. Faltinsen, Sea Loads on Ships and Offshore
% Structures (1990).
%
% Open issues: - 2nd-order wave forces in heave? Drift?
%              - Include speed effect => change of frequency?
%              - Moment (drift) is simply: M = f*r = f * integral_from_front_to_end
%              - Diffraction loads depend on hydrodynamic added mass. Not
%               implemented
%

%% Initial parameters
clear all; close all; clc;
tStart = 0; tFinal = 120; tStep = .2;

T = tStart:tStep:tFinal;
% rand('twister',1501);

plotForces = 1; % Plot forces/moments in 6DOF
shortCrest = 0; % Choose between long and short crested waves simulation
DOF = 3;

%% Generate wave spectrum
waveDir = (45)*pi/180; % Direction of incoming waves relative to x-axis

%% Loop & Animation, Time Series

% Init. waveForce vectors contain forces and moments.
waveElevation = zeros(2,length(T));
waveForceDrift = zeros(DOF,length(T));
waveForceFK = zeros(DOF,length(T));
waveForceTotal = zeros(DOF,length(T));

%%
for j = 1:length(T)
    t = T(j);
    [waveForceTotal(:,j),waveForceDrift(:,j),waveForceFK(:,j)] = waveForces(t,waveDir,j-1,DOF,shortCrest,tFinal);
end

%% Force Plots
if plotForces
    figure;
    plot(T,waveForceTotal(1,:),'r',T,waveForceFK(1,:),'g:',T,waveForceDrift(1,:),'b','LineWidth',2);  grid on;
    title('Wave Forces Time Series (surge)'); xlabel('Time [s]'); ylabel('Wave Force [kN]');
    legend('Total','Froude-Krilov (1st-order)','Mean Drift (2nd-order)');

    figure;
    plot(T,waveForceTotal(2,:),'r',T,waveForceFK(2,:),'g:',T,waveForceDrift(2,:),'b','LineWidth',2);  grid on;
    title('Wave Forces Time Series (sway)'); xlabel('Time [s]'); ylabel('Wave Force [kN]');
    legend('Total','Froude-Krilov (1st-order)','Mean Drift (2nd-order)');
    if DOF == 3
        figure;
        plot(T,waveForceTotal(3,:),'r',T,waveForceFK(3,:),'g:',T,waveForceDrift(3,:),'b','LineWidth',2);  grid on;
        title('Wave Forces Time Series (yaw)'); xlabel('Time [s]'); ylabel('Wave Moment [kNm]');
        legend('Total','Froude-Krilov (1st-order)','Mean Drift (2nd-order)');
    
    elseif DOF == 6
        figure;
        plot(T,waveForceTotal(3,:),'r',T,waveForceFK(3,:),'g:',T,waveForceDrift(3,:),'b','LineWidth',2);  grid on;
        title('Wave Forces Time Series (heave)'); xlabel('Time [s]'); ylabel('Wave Force [kN]');
        legend('Total','Froude-Krilov (1st-order)','Mean Drift (2nd-order)');

        figure;
        plot(T,waveForceTotal(4,:),'r',T,waveForceFK(4,:),'g:',T,waveForceDrift(4,:),'b','LineWidth',2);  grid on;
        title('Wave Forces Time Series (roll)'); xlabel('Time [s]'); ylabel('Wave Moment [kNm]');
        legend('Total','Froude-Krilov (1st-order)','Mean Drift (2nd-order)');

        figure;
        plot(T,waveForceTotal(5,:),'r',T,waveForceFK(5,:),'g:',T,waveForceDrift(5,:),'b','LineWidth',2);  grid on;
        title('Wave Forces Time Series (pitch)'); xlabel('Time [s]'); ylabel('Wave Moment [kNm]');
        legend('Total','Froude-Krilov (1st-order)','Mean Drift (2nd-order)');

        figure;
        plot(T,waveForceTotal(6,:),'r',T,waveForceFK(6,:),'g:',T,waveForceDrift(6,:),'b','LineWidth',2);  grid on;
        title('Wave Forces Time Series (yaw)'); xlabel('Time [s]'); ylabel('Wave Moment [kNm]');
        legend('Total','Froude-Krilov (1st-order)','Mean Drift (2nd-order)');
    end
end